@@ -93,16 +93,19 @@ class BleDevice(NrfDriverObserver):
     configuration, and bond database.
 
     :param comport: The port the nRF52 device lives on, e.g. ``"COM3"``, ``"/dev/ttyS0"``
-    :param baud: The baud rate to use. By default the connectivity firmware images for v0.3+ use 1M baud.
+    :param baud: The baud rate to use. By default, the connectivity firmware images for v0.3+ use 1M baud.
     :param log_driver_comms: debug flag which will enable extra-verbose logging of all communications to the nRF52 hardware
     :param notification_hw_queue_size: Hardware-based queue size to use for notifications.
                                        This queue lives within the nRF52 hardware itself and has memory usage implications based on MTU size, etc.
                                        This probably won't need to be changed, and from current testing the queue isn't fully exercised
     :param write_command_hw_queue_size: Hardware-based queue size to use for write commands (write w/o response).
                                         Same comments about notification queue apply here.
-    :param bond_db_filename: Optional filename to use for loading/saving the bonding database. Two special values also exist:
+    :param bond_db_filename: Optional filename to use for loading/saving the bonding database.
+                             The supported file formats/extensions are: ".pkl" (legacy) and ".json". json is preferred.
 
-                             - ``"user"`` [default] - saves the database within the user's home directory  (``~/.blatann/bonding_db.pkl``).
+                             Two special values also exist:
+
+                             - ``"user"`` [default] - saves the database within the user's home directory  (``~/.blatann/bonding_db.json``).
                                This is useful for cases where you may not have write access to the python install location, want to
                                persist the bonding database across virtualenvs, or limit the access to just the logged-in user
                              - ``"system"`` - saves the database within this library's directory structure, wherever it is installed or imported from.
@@ -121,11 +124,7 @@ class BleDevice(NrfDriverObserver):
                                                             hvn_tx_queue_size=notification_hw_queue_size,         # Hardware queue of 16 notifications
                                                             write_cmd_tx_queue_size=write_command_hw_queue_size)  # Hardware queue of 16 write cmds (no response)
 
-        special_bond_db_filemap = {"user": default_bond_db.user_default_db_file, "system": default_bond_db.system_default_db_file}
-        bond_db_filename = special_bond_db_filemap.get(bond_db_filename.lower(), bond_db_filename)
-
-        self.bond_db_loader = default_bond_db.DefaultBondDatabaseLoader(bond_db_filename)
-        self.bond_db = default_bond_db.DefaultBondDatabase()
+        self._setup_bond_db(bond_db_filename)
 
         self.client = peer.Client(self)
         self.connected_peripherals = {}
@@ -140,13 +139,18 @@ class BleDevice(NrfDriverObserver):
         self._default_security_params = peer.DEFAULT_SECURITY_PARAMS
         self._att_mtu_max = MTU_SIZE_MINIMUM
 
+    def _setup_bond_db(self, bond_db_filename):
+        self.bond_db_loader = default_bond_db.DefaultBondDatabaseLoader(bond_db_filename)
+        self.bond_db = default_bond_db.DefaultBondDatabase()
+
     def configure(self, vendor_specific_uuid_count=10,
                   service_changed=False,
                   max_connected_peripherals=1,
                   max_connected_clients=1,
                   max_secured_peripherals=1,
                   attribute_table_size=nrf_types.driver.BLE_GATTS_ATTR_TAB_SIZE_DEFAULT,
-                  att_mtu_max_size=MTU_SIZE_FOR_MAX_DLE):
+                  att_mtu_max_size=MTU_SIZE_FOR_MAX_DLE,
+                  event_length=6):
         """
         Configures the BLE Device with the given settings.
 
@@ -155,7 +159,7 @@ class BleDevice(NrfDriverObserver):
         :param vendor_specific_uuid_count: The Nordic hardware limits number of 128-bit Base UUIDs
                                            that the device can know about. This normally equals the number of custom services
                                            that are to be supported, since characteristic UUIDs are usually derived from the service base UUID.
-        :param service_changed: Whether or not the Service Changed characteristic is exposed in the GAP service
+        :param service_changed: Whether the Service Changed characteristic is exposed in the GAP service
         :param max_connected_peripherals: The maximum number of concurrent connections with peripheral devices
         :param max_connected_clients: The maximum number of concurrent connections with client devices (NOTE: blatann currently only supports 1)
         :param max_secured_peripherals: The maximum number of concurrent peripheral connections that will need security (bonding/pairing) enabled
@@ -163,14 +167,20 @@ class BleDevice(NrfDriverObserver):
                                      Increase this number if there's a lot of services/characteristics in your GATT database.
         :param att_mtu_max_size: The maximum ATT MTU size supported. The default supports an MTU which will fit into
                                  a single transmission if Data Length Extensions is set to its max (251)
+        :param event_length: The number of 1.25ms event cycles to dedicate for each connection.
+                             The default value (6, =7.5ms) will support the max DLE length of 251 bytes.
+                             Minimum value is 2, typical values are 3-8 depending on desired throughput
+                             and number of concurrent connections
         """
         if self.ble_driver.is_open:
             raise exceptions.InvalidStateException("Cannot configure the BLE device after it has been opened")
-
+        if event_length < nrf_types.driver.BLE_GAP_EVENT_LENGTH_MIN:
+            raise ValueError(f"Event length must be >= {nrf_types.driver.BLE_GAP_EVENT_LENGTH_MIN}")
         self._ble_configuration = nrf_types.BleEnableConfig(vendor_specific_uuid_count, max_connected_clients,
                                                             max_connected_peripherals, max_secured_peripherals,
                                                             service_changed, attribute_table_size)
         self._default_conn_config.max_att_mtu = att_mtu_max_size
+        self._default_conn_config.event_length = event_length
 
     def open(self, clear_bonding_data=False):
         """
@@ -330,6 +340,21 @@ class BleDevice(NrfDriverObserver):
         self._default_security_params = SecurityParameters(passcode_pairing, io_capabilities, bond, out_of_band,
                                                            reject_pairing_requests, lesc_pairing)
 
+    def set_privacy_settings(self, enabled: bool, resolvable_address: bool = True,
+                             update_rate_seconds: int = nrf_types.BLEGapPrivacyParams.DEFAULT_PRIVATE_ADDR_CYCLE_INTERVAL_S):
+        """
+        Sets the privacy parameters for advertising and connections to the device. When enabled, a random private address
+        will be advertised and updated at the provided interval.
+
+        :param enabled: True to enable device privacy. Note that only device privacy is supported, network privacy is not
+        :param resolvable_address: True to use a private random resolvable address.
+                                   If the address is resolvable, bonded peers can use the device's IRK to determine the
+                                   device's actual public/random address.
+        :param update_rate_seconds: How often the address should be changed/updated, in seconds. Default is 900 (15min)
+        """
+        params = nrf_types.BLEGapPrivacyParams(enabled, resolvable_address, update_rate_seconds)
+        self.ble_driver.ble_gap_privacy_set(params)
+
     def _on_user_mem_request(self, nrf_driver, event):
         # Only action that can be taken
         self.ble_driver.ble_user_mem_reply(event.conn_handle)
