@@ -330,6 +330,13 @@ class NrfDriver(object):
     def ble_gap_tx_power_set(self, tx_power):
         return driver.sd_ble_gap_tx_power_set(self.rpc_adapter, tx_power)
 
+    @NordicSemiErrorCheck
+    @wrapt.synchronized
+    def ble_gap_privacy_set(self, privacy):
+        assert isinstance(privacy, BLEGapPrivacyParams)
+        params = privacy.to_c()
+        return driver.sd_ble_gap_privacy_set(self.rpc_adapter, params)
+
     @NordicSemiErrorCheck
     @wrapt.synchronized
     def ble_gap_adv_start(self, adv_params=None, conn_cfg_tag=0):
