@@ -2,13 +2,15 @@ from __future__ import annotations
 import logging
 import threading
 import enum
+from typing import Optional, Type
 
 from blatann.event_type import EventSource, Event
 from blatann.gap import smp
+from blatann.gap.gap_types import Phy, PeerAddress, ConnectionParameters, ActiveConnectionParameters
 from blatann.gatt import gattc, service_discovery, MTU_SIZE_DEFAULT, MTU_SIZE_MINIMUM, DLE_MAX, DLE_MIN, DLE_OVERHEAD
 from blatann.nrf import nrf_events
 from blatann.nrf.nrf_types.enums import BLE_CONN_HANDLE_INVALID
-from blatann.nrf.nrf_types import conn_interval_range, conn_timeout_range, BLEGapDataLengthParams
+from blatann.nrf.nrf_types import BLEGapDataLengthParams
 from blatann.waitables.waitable import EmptyWaitable
 from blatann.waitables.connection_waitable import DisconnectionWaitable
 from blatann.waitables.event_waitable import EventWaitable
@@ -26,87 +28,6 @@ class PeerState(enum.Enum):
     CONNECTED = 2     #: Peer is connected
 
 
-class Phy(enum.IntFlag):
-    """
-    The supported PHYs
-
-    .. note:: Coded PHY is currently not supported (hardware limitation)
-    """
-    auto = int(nrf_events.BLEGapPhy.auto)          #: Automatically select the PHY based on what's supported
-    one_mbps = int(nrf_events.BLEGapPhy.one_mbps)  #: 1 Mbps PHY
-    two_mbps = int(nrf_events.BLEGapPhy.two_mbps)  #: 2 Mbps PHY
-
-    # NOT SUPPORTED coded = int(nrf_events.BLEGapPhy.coded)
-
-
-class PeerAddress(nrf_events.BLEGapAddr):
-    pass
-
-
-class ConnectionParameters(nrf_events.BLEGapConnParams):
-    """
-    Represents the connection parameters that are sent during negotiation. This includes
-    the preferred min/max interval range, timeout, and slave latency
-    """
-    def __init__(self, min_conn_interval_ms, max_conn_interval_ms, timeout_ms, slave_latency=0):
-        super(ConnectionParameters, self).__init__(min_conn_interval_ms, max_conn_interval_ms, timeout_ms, slave_latency)
-        self.validate()
-
-    def validate(self):
-        conn_interval_range.validate(self.min_conn_interval_ms)
-        conn_interval_range.validate(self.max_conn_interval_ms)
-        conn_timeout_range.validate(self.conn_sup_timeout_ms)
-        if self.min_conn_interval_ms > self.max_conn_interval_ms:
-            raise ValueError(f"Minimum connection interval must be <= max connection interval "
-                             f"(Min: {self.min_conn_interval_ms} Max: {self.max_conn_interval_ms}")
-
-
-class ActiveConnectionParameters(object):
-    """
-    Represents the connection parameters that are currently in use with a peer device.
-    This is similar to ConnectionParameters with the sole difference being
-    the connection interval is not a min/max range but a single number
-    """
-    def __init__(self, conn_params: ConnectionParameters):
-        self._interval_ms = conn_params.min_conn_interval_ms
-        self._timeout_ms = conn_params.conn_sup_timeout_ms
-        self._slave_latency = conn_params.slave_latency
-
-    def __repr__(self):
-        return str(self)
-
-    def __str__(self):
-        return f"ConnectionParams({self._interval_ms}ms/{self._slave_latency}/{self._timeout_ms}ms)"
-
-    @property
-    def interval_ms(self) -> float:
-        """
-        **Read Only**
-
-        The connection interval, in milliseconds
-        """
-        return self._interval_ms
-
-    @property
-    def timeout_ms(self) -> float:
-        """
-        **Read Only**
-
-        The connection timeout, in milliseconds
-        """
-        return self._timeout_ms
-
-    @property
-    def slave_latency(self) -> int:
-        """
-        **Read Only**
-
-        The slave latency (the number of connection intervals the slave is allowed to skip before being
-        required to respond)
-        """
-        return self._slave_latency
-
-
 DEFAULT_CONNECTION_PARAMS = ConnectionParameters(15, 30, 4000, 0)
 DEFAULT_SECURITY_PARAMS = smp.SecurityParameters(reject_pairing_requests=True)
 
@@ -131,6 +52,7 @@ class Peer(object):
         self._name = name
         self._preferred_connection_params = connection_params
         self._current_connection_params = ActiveConnectionParameters(connection_params)
+        self._rssi_report_started = False
         self.conn_handle = BLE_CONN_HANDLE_INVALID
         self.peer_address = "",
         self.connection_state = PeerState.DISCONNECTED
@@ -138,8 +60,10 @@ class Peer(object):
         self._on_disconnect = EventSource("On Disconnect", logger)
         self._on_mtu_exchange_complete = EventSource("On MTU Exchange Complete", logger)
         self._on_mtu_size_updated = EventSource("On MTU Size Updated", logger)
+        self._on_conn_params_updated = EventSource("On Connection Parameters Updated", logger)
         self._on_data_length_updated = EventSource("On Data Length Updated", logger)
         self._on_phy_updated = EventSource("On Phy Updated", logger)
+        self._on_rssi_changed = EventSource("On RSSI Updated", logger)
         self._mtu_size = MTU_SIZE_DEFAULT
         self._preferred_mtu_size = MTU_SIZE_DEFAULT
         self._negotiated_mtu_size = None
@@ -162,7 +86,7 @@ class Peer(object):
     def name(self) -> str:
         """
         The name of the peer, if known. This property is for the user's benefit to name certain connections.
-        The name is is also saved in the case that the peer is subsequently bonded to and can be looked up that way
+        The name is also saved in the case that the peer is subsequently bonded to and can be looked up that way
         in the bond database
 
         .. note:: For central peers this name is unknown unless set by the setter.
@@ -186,6 +110,19 @@ class Peer(object):
         """
         return self.connection_state == PeerState.CONNECTED
 
+    @property
+    def rssi(self) -> Optional[int]:
+        """
+        **Read Only**
+
+        Gets the RSSI from the latest connection interval, or None if RSSI reporting is not enabled.
+
+        .. note:: In order for RSSI information to be available, :meth:`start_rssi_reporting` must be called first.
+        """
+        if not self._rssi_report_started:
+            return None
+        return self._ble_device.ble_driver.ble_gap_rssi_get(self.conn_handle)
+
     @property
     def bytes_per_notification(self) -> int:
         """
@@ -218,7 +155,8 @@ class Peer(object):
         """
         **Read Only**
 
-        Gets if the peer has bonding information stored in the bond database (the peer was bonded to in a previous connection)
+        Gets if the peer has bonding information stored in the bond database
+        (the peer was bonded to in a previous connection)
         """
         return self.security.is_previously_bonded
 
@@ -263,7 +201,8 @@ class Peer(object):
     def preferred_mtu_size(self) -> int:
         """
         The user-set preferred MTU size. Defaults to the Bluetooth default MTU size (23).
-        This is the value that will be negotiated during an MTU Exchange but is not guaranteed in the case that the peer has a smaller MTU
+        This is the value that will be negotiated during an MTU Exchange
+        but is not guaranteed in the case that the peer has a smaller MTU
 
         :getter: Gets the preferred MTU size that was configured
         :setter: Sets the preferred MTU size to use for MTU exchanges
@@ -331,6 +270,13 @@ class Peer(object):
         """
         return self._on_disconnect
 
+    @property
+    def on_rssi_changed(self) -> Event[Peer, int]:
+        """
+        Event generated when the RSSI has changed for the connection
+        """
+        return self._on_rssi_changed
+
     @property
     def on_mtu_exchange_complete(self) -> Event[Peer, MtuSizeUpdatedEventArgs]:
         """
@@ -345,6 +291,13 @@ class Peer(object):
         """
         return self._on_mtu_size_updated
 
+    @property
+    def on_connection_parameters_updated(self) -> Event[Peer, ConnectionParametersUpdatedEventArgs]:
+        """
+        Event generated when the connection parameters with this peer is updated
+        """
+        return self._on_conn_params_updated
+
     @property
     def on_data_length_updated(self) -> Event[Peer, DataLengthUpdatedEventArgs]:
         """
@@ -388,29 +341,38 @@ class Peer(object):
                                   min_connection_interval_ms: float,
                                   max_connection_interval_ms: float,
                                   connection_timeout_ms: int,
-                                  slave_latency=0):
+                                  slave_latency=0) -> Optional[EventWaitable[Peer, ConnectionParametersUpdatedEventArgs]]:
         """
         Sets the connection parameters for the peer and starts the connection parameter update process (if connected)
 
-        .. note:: Connection interval values should be a multiple of 1.25ms since that is the granularity allowed in the Bluetooth specification.
-           Any non-multiples will be rounded down to the nearest 1.25ms.
+        .. note:: Connection interval values should be a multiple of 1.25ms since that is the granularity allowed
+           in the Bluetooth specification. Any non-multiples will be rounded down to the nearest 1.25ms.
            Additionally, the connection timeout has a granularity of 10 milliseconds and will also be rounded as such.
 
         :param min_connection_interval_ms: The minimum acceptable connection interval, in milliseconds
         :param max_connection_interval_ms: The maximum acceptable connection interval, in milliseconds
         :param connection_timeout_ms: The connection timeout, in milliseconds
-        :param slave_latency: The slave latency allowed, which regulates how many connection intervals the peripheral is allowed to skip before responding
+        :param slave_latency: The slave latency allowed, which regulates how many connection intervals
+                              the peripheral is allowed to skip before responding
+
+        :return: If the peer is connected, this will return a waitable that will trigger when the update completes
+                 with the new connection parameters. If disconnected, returns None
         """
         self._preferred_connection_params = ConnectionParameters(min_connection_interval_ms, max_connection_interval_ms,
                                                                  connection_timeout_ms, slave_latency)
         if self.connected:
-            self.update_connection_parameters()
+            return self.update_connection_parameters()
+        return None
 
-    def update_connection_parameters(self):
+    def update_connection_parameters(self) -> EventWaitable[Peer, ConnectionParametersUpdatedEventArgs]:
         """
-        Starts the process to re-negotiate the connection parameters using the previously-set connection parameters
+        Starts the process to re-negotiate the connection parameters
+        using the configured preferred connection parameters
+
+        :return: A waitable that will trigger when the connection parameters are updated
         """
         self._ble_device.ble_driver.ble_gap_conn_param_update(self.conn_handle, self._preferred_connection_params)
+        return EventWaitable(self._on_conn_params_updated)
 
     def exchange_mtu(self, mtu_size=None) -> EventWaitable[Peer, MtuSizeUpdatedEventArgs]:
         """
@@ -480,6 +442,31 @@ class Peer(object):
         self._discoverer.start()
         return EventWaitable(self._discoverer.on_discovery_complete)
 
+    def start_rssi_reporting(self, threshold_dbm: int = None, skip_count=1) -> EventWaitable[Peer, int]:
+        """
+        Starts collecting RSSI readings for the connection
+
+        :param threshold_dbm: Minimum change in dBm before triggering an RSSI changed event.
+                              The default value ``None`` disables the RSSI event
+                              (RSSI polled via the :attr:`rssi` property)
+        :param skip_count: Number of RSSI samples with a change of threshold_dbm or more before
+                           sending a new RSSI update event. Parameter ignored if threshold_dbm is None
+        :return: a Waitable that triggers once the first RSSI value is received, if threshold_dbm is not None
+        """
+        if self._rssi_report_started:
+            self.stop_rssi_reporting()
+        waitable = EventWaitable(self.on_rssi_changed)
+        self._ble_device.ble_driver.ble_gap_rssi_start(self.conn_handle, threshold_dbm, skip_count)
+        self._rssi_report_started = True
+        return waitable
+
+    def stop_rssi_reporting(self):
+        """
+        Stops collecting RSSI readings. Once stopped, :attr:`rssi` will return ``None``
+        """
+        self._ble_device.ble_driver.ble_gap_rssi_stop(self.conn_handle)
+        self._rssi_report_started = False
+
     """
     Internal Library Methods
     """
@@ -488,24 +475,26 @@ class Peer(object):
         """
         Internal method called when the peer connects to set up the object.
 
-        **Should not be called by the user**
+        :meta private:
         """
         self.conn_handle = conn_handle
         self.peer_address = peer_address
         self._mtu_size = MTU_SIZE_DEFAULT
         self._negotiated_mtu_size = None
+        self._rssi_report_started = False
         self._disconnect_waitable = DisconnectionWaitable(self)
         self.connection_state = PeerState.CONNECTED
         self._current_connection_params = ActiveConnectionParameters(connection_params)
 
         self._ble_device.ble_driver.event_subscribe(self._on_disconnect_event, nrf_events.GapEvtDisconnected)
-        self.driver_event_subscribe(self._on_connection_param_update, nrf_events.GapEvtConnParamUpdate, nrf_events.GapEvtConnParamUpdateRequest)
+        self.driver_event_subscribe(self._on_connection_param_update, nrf_events.GapEvtConnParamUpdate)
         self.driver_event_subscribe(self._on_mtu_exchange_request, nrf_events.GattsEvtExchangeMtuRequest)
         self.driver_event_subscribe(self._on_mtu_exchange_response, nrf_events.GattcEvtMtuExchangeResponse)
         self.driver_event_subscribe(self._on_data_length_update_request, nrf_events.GapEvtDataLengthUpdateRequest)
         self.driver_event_subscribe(self._on_data_length_update, nrf_events.GapEvtDataLengthUpdate)
         self.driver_event_subscribe(self._on_phy_update_request, nrf_events.GapEvtPhyUpdateRequest)
         self.driver_event_subscribe(self._on_phy_update, nrf_events.GapEvtPhyUpdate)
+        self.driver_event_subscribe(self._rssi_changed, nrf_events.GapEvtRssiChanged)
         self._on_connect.notify(self)
 
     def _check_driver_event_connection_handle_wrapper(self, func):
@@ -519,6 +508,7 @@ class Peer(object):
         Internal method that subscribes handlers to NRF Driver events directed at this peer.
         Handlers are automatically unsubscribed once the peer disconnects.
 
+        :meta private:
         :param handler: The handler to subscribe
         :param event_types: The NRF Driver event types to subscribe to
         """
@@ -532,6 +522,7 @@ class Peer(object):
         """
         Internal method that unsubscribes handlers from NRF Driver events.
 
+        :meta private:
         :param handler: The handler to unsubscribe
         :param event_types: The event types to unsubscribe from
         """
@@ -546,10 +537,7 @@ class Peer(object):
     Private Methods
     """
 
-    def _on_disconnect_event(self, driver, event):
-        """
-        :type event: nrf_events.GapEvtDisconnected
-        """
+    def _on_disconnect_event(self, driver, event: nrf_events.GapEvtDisconnected):
         if not self.connected or self.conn_handle != event.conn_handle:
             return
         self.conn_handle = BLE_CONN_HANDLE_INVALID
@@ -564,18 +552,15 @@ class Peer(object):
         self._ble_device.ble_driver.event_unsubscribe(self._on_disconnect_event)
         self._ble_device.ble_driver.event_unsubscribe(self._on_connection_param_update)
 
-    def _on_connection_param_update(self, driver, event):
-        """
-        :type event: nrf_events.GapEvtConnParamUpdate
-        """
-        if not self.connected or self.conn_handle != event.conn_handle:
+    def _on_connection_param_update(self, driver, event: nrf_events.GapEvtConnParamUpdate):
+        if not self.connected:
             return
-        if isinstance(event, nrf_events.GapEvtConnParamUpdateRequest):
-            logger.debug("[{}] Conn Params updating to {}".format(self.conn_handle, self._preferred_connection_params))
-            self._ble_device.ble_driver.ble_gap_conn_param_update(self.conn_handle, self._preferred_connection_params)
-        else:
-            logger.debug("[{}] Updated to {}".format(self.conn_handle, event.conn_params))
-            self._current_connection_params = ActiveConnectionParameters(event.conn_params)
+        logger.debug("[{}] Conn params updated: {}".format(self.conn_handle, event.conn_params))
+        self._current_connection_params = ActiveConnectionParameters(event.conn_params)
+        self._on_conn_params_updated.notify(self, ConnectionParametersUpdatedEventArgs(self._current_connection_params))
+
+    def _rssi_changed(self, driver, event: nrf_events.GapEvtRssiChanged):
+        self._on_rssi_changed.notify(self, event.rssi)
 
     def _validate_mtu_size(self, mtu_size):
         if mtu_size < MTU_SIZE_MINIMUM:
@@ -629,7 +614,8 @@ class Peer(object):
 
 class Peripheral(Peer):
     """
-    Object which represents a BLE-connected device that is acting as a peripheral/server (local device is client/central)
+    Object which represents a BLE-connected device that is acting as a peripheral/server
+    (local device is client/central)
     """
     def __init__(self, ble_device, peer_address,
                  connection_params=DEFAULT_CONNECTION_PARAMS,
@@ -640,11 +626,66 @@ class Peripheral(Peer):
                                          security_params, name, write_no_resp_queue_size)
         self.peer_address = peer_address
         self.connection_state = PeerState.CONNECTING
+        self._conn_param_update_request_handler = self._accept_all_conn_param_requests
+
+    def set_conn_param_request_handler(self, handler: TConnectionParamUpdateRequestHandler):
+        """
+        Configures a function callback to handle when a connection parameter request is received from the peripheral
+        and allows the user to decide how to handle the peripheral's requested connection parameters.
+
+        The callback is passed in 2 positional parameters: this ``Peripheral`` object
+        and the desired ``ConnectionParameter``s received in the request.
+        The callback should return the desired connection parameters to use, or None to reject the request altogether.
+
+        :param handler: The callback to determine which connection parameters to negotiate when an update request
+                        is received from the peripheral
+        """
+        self._conn_param_update_request_handler = handler
+
+    def accept_all_conn_param_requests(self):
+        """
+        Sets the connection parameter request handler to a callback that accepts any connection parameter
+        update requests received from the peripheral. This is the same as calling ``set_conn_param_request_handler``
+        with a callback that simply returns the connection parameters passed in.
+
+        This is the default functionality.
+        """
+        self._conn_param_update_request_handler = self._accept_all_conn_param_requests
+
+    def reject_conn_param_requests(self):
+        """
+        Sets the connection parameter request handler to a callback that rejects all connection parameter
+        update requests received from the peripheral. This is same as calling ``set_conn_param_request_handler``
+        with a callback that simply returns ``None``
+        """
+        self._conn_param_update_request_handler = self._reject_all_conn_param_requests
+
+    def peer_connected(self, conn_handle, peer_address, connection_params):
+        """
+        :meta private:
+        """
+        self.driver_event_subscribe(self._on_connection_param_update_request, nrf_events.GapEvtConnParamUpdateRequest)
+        super(Peripheral, self).peer_connected(conn_handle, peer_address, connection_params)
+
+    def _accept_all_conn_param_requests(self, peer: Peripheral, conn_params: ConnectionParameters):
+        return conn_params
+
+    def _reject_all_conn_param_requests(self, peer: Peripheral, conn_params: ConnectionParameters):
+        return None
+
+    def _on_connection_param_update_request(self, driver, event: nrf_events.GapEvtConnParamUpdateRequest):
+        if not self.connected:
+            return
+        conn_params = self._conn_param_update_request_handler(self, event.conn_params)
+        logger.debug("[{}] Conn params update request to: {}. Return: {}".format(self.conn_handle, event.conn_params, conn_params))
+
+        self._ble_device.ble_driver.ble_gap_conn_param_update(self.conn_handle, conn_params)
 
 
 class Client(Peer):
     """
-    Object which represents a BLE-connected device that is acting as a client/central (local device is peripheral/server)
+    Object which represents a BLE-connected device that is acting as a client/central
+    (local device is peripheral/server)
     """
     def __init__(self, ble_device,
                  connection_params=DEFAULT_CONNECTION_PARAMS,
@@ -656,6 +697,9 @@ class Client(Peer):
         self._first_connection = True
 
     def peer_connected(self, conn_handle, peer_address, connection_params):
+        """
+        :meta private:
+        """
         # Recreate the DB and discovery class since the client object persists across disconnects
         if not self._first_connection:
             self._db = gattc.GattcDatabase(self._ble_device, self)
@@ -663,3 +707,9 @@ class Client(Peer):
         self._first_connection = False
         self._name = ""
         super(Client, self).peer_connected(conn_handle, peer_address, connection_params)
+
+
+# Type alias for callback function which handles connection parameter update requests.
+# Function is passed in a Peripheral and ConnectionParameters object
+# and returns the negotiated ConnectionParameters, or None to reject
+TConnectionParamUpdateRequestHandler: Type = Callable[[Peripheral, ConnectionParameters], Optional[ConnectionParameters]]
