@@ -1,16 +1,13 @@
-from typing import TypeVar, Generic, Tuple
+from typing import Generic, Tuple, Callable
 from blatann.waitables.waitable import Waitable
-
-
-TSender = TypeVar("TSender")
-TEvent = TypeVar("TEvent")
+from blatann.event_type import Event, TSender, TEvent
 
 
 class EventWaitable(Waitable, Generic[TSender, TEvent]):
-    def __init__(self, event):
-        """
-        :type event: blatann.event_type.Event
-        """
+    """
+    Waitable implementation which waits on an :class:`~blatann.event_type.Event`.
+    """
+    def __init__(self, event: Event[TSender, TEvent]):
         super(EventWaitable, self).__init__(n_args=2)
         self._event = event
         self._event.register(self._on_event)
@@ -28,11 +25,18 @@ class EventWaitable(Waitable, Generic[TSender, TEvent]):
             return None, None
         return res
 
+    def then(self, callback: Callable[[TSender, TEvent], None]):
+        return super(EventWaitable, self).then(callback)
 
-class IdBasedEventWaitable(EventWaitable):
+
+class IdBasedEventWaitable(EventWaitable, Generic[TSender, TEvent]):
+    """
+    Extension of :class:`EventWaitable` for high-churn events which require IDs to ensure the correct operation is waited upon,
+    such as characteristic read, write and notify operations
+    """
     def __init__(self, event, event_id):
-        super(IdBasedEventWaitable, self).__init__(event)
         self.id = event_id
+        super(IdBasedEventWaitable, self).__init__(event)
 
     def _on_event(self, sender, event_args):
         if event_args.id == self.id:
